#include <boost/asio.hpp>
#include <boost/bind.hpp>
#include <boost/date_time/posix_time/posix_time.hpp>
#include <boost/asio/serial_port.hpp>
//#include <boost/lambda/lambda.hpp>
#include <iostream>
#include <iterator>
#include <map>
//#include <algorithm>

#include "utils.h"
#include "http/server.hpp"
#include "http_json_handler.h"
#include "zextension.h"

using namespace std;

using namespace ZRouter;

static ZRouter::T_ZExtensionMap extension_init_map;

ZExtensionManager::ZExtensionManager() {
	cout << "ZExtensionManager::" << endl;
}

void ZExtensionManager::register_extension(string mod_name, FP_ExtensionInitializer ext_initiator) {
	extension_init_map[mod_name] = ext_initiator;
}


int main()
{
	// Make single IO handler
  boost::asio::io_service io;

	// Allocate object model
	Model::Model model;

	// and fill with data
	model.populate();

	// Create our custom http model interface handler
	http_json_handler json_handler(model);

	// Start server
	http::server::server s(io, "0.0.0.0", "9080", "html");

	// Register model interface	
	s.register_handler(&json_handler); 

	cout << "Loading extensions" << endl; 
	// Add all available extensions
	ZRouter::T_ZExtensionMap::iterator ext_ptr = extension_init_map.begin();
	while (ext_ptr != extension_init_map.end()) {
		(*ext_ptr).second(io, model);
		ext_ptr++;
	}
	cout << "Loading extensions completed" << endl; 
 
	io.run();

  return 0;
}

